import numpy as np
import os

import pyts
import pyts.runInput.main as ptsm
import pyts.io.input as ptsin
from pyts.base import tsGrid
from pyts.phaseModels.main import Rinker, Uniform


from wisdem.aeroelasticse.CaseGen_General import CaseGen_General, save_case_matrix

# class pyTurbsim_wrapper():
#     """ wrapper for py turbsim so we can mostly just plug it in """
#     def __init__(self):
#         import pyts
#         import pyts.runInput.main as ptsm
#         import pyts.io.input as ptsin
#         from pyts.base import tsGrid
#         from pyts.phaseModels.main import Rinker, Uniform

#         pass

#     def execute(self, IEC_WindType, U, rho):  ### Note rho is a new param.  maybe a dict as input would be better,
#         ### but that would mean changing pyIECWind, or at least adding a new function like "execute_dict()"
#         pass


class pyTurbsim_wrapper():
    """ a component to run TurbSim.
        will try to make it aware of whether the wind file already exists"""

    def create_tsr(self, rho, U, tmax, dt):
        ######## NOT CALLED ##############
        tsr = api.tsrun()
        tsr.grid = api.tsGrid(center=10, ny=self.ny, nz=self.nz,
                              height=10, width=10, time_sec=tmax, dt=dt)
        tsr.prof = api.profModels.pl(U, 90)

        tsr.spec = api.specModels.tidal(self.ustar, 10)

        #tsr.cohere = api.cohereModels.nwtc()
        tsr.cohere = pyts.cohereModels.main.none()

        tsr.stress = api.stressModels.uniform(0.0, 0.0, 0.0)

        tsr.cohere = pyts.cohereModels.main.none()
        tsr.stress = pyts.stressModels.main.uniform(0,0,0)

        from pyts.phaseModels.main import Rinker, Uniform
        tsr.phase = Rinker(rho, self.mu)
        #tsr.phase = Uniform()
        # tsr.stress=np.zeros(tsr.grid.shape,dtype='float32')

        self.tsr = tsr
        return tsr
        #######################################


    def __init__(self, filedict, case, case_name):

        self.overwrite = True

        self.case = case
        self.case_name = case_name

#        self.rawts.ts_exe = filedict['ts_exe']
        self.ts_dir = filedict['ts_dir']
        self.ts_file = filedict['ts_file']

#        self.ustar = .8
##        self.U = 17.
#        self.ny = 15
#        self.nz = 15
#        self.dt = 0.05  # these (should/have in past) come from TurbSim template file
#        self.mid = self.nz/2

##        self.rho = 0.9999
        self.mu = np.pi

###        np.random.seed(1)  ## probably don't want this in this context

#        self.rawts.run_name = self.run_name

        try:
            self.basedir = os.path.join(os.getcwd(),"allts_runs")
        except:
            pass
        
        if 'run_dir' in filedict:
            try:
                self.basedir = os.path.join(os.getcwd(),filedict['run_dir'])
            except:
                self.basedir = filedict['run_dir']
        if (not os.path.exists(self.basedir)):
            try:
                os.mkdir(self.basedir)
            except:
                pass
                # print("ok, %s exists after all" % self.basedir)

    def add_phase_dist(self, tsr, rho, tmax):
        tsr.cohere = pyts.cohereModels.main.none()
        #tsr.cohere = pyts.cohereModels.main.nwtc()
        #tsr.stress = pyts.stressModels.main.uniform(0,0,0)
    ####    tsr.phase = Rinker(rho, self.mu)  ### pgraf turned it off for testing!
    ####    cg = tsr.grid
    ###    tsr.grid = tsGrid(center=cg.center, ny=cg.n_y, nz=cg.n_z,
    ###                      height=cg.height, width=cg.width,
    ###                      time_sec=tmax, dt=cg.dt)

        return tsr

    def execute(self):
        case = self.case
        case_name = self.case_name
        # print("CASE", case, case_name)
        ws=case['Vhub']
        rho = case['Rho']   #case.fst_params['rho'] ####### TODO: how does this get here?
        rs = case['RandSeed1'] if 'RandSeed1' in case else None
        tmax = 2  ## should not be hard default ##
        if ('TMax' in case):  ## Note, this gets set via "AnalTime" in input files--FAST peculiarity ? ##
            tmax = case['TMax']

        # run TurbSim to generate the wind:
        run_dir = os.path.join(self.basedir, case_name)
        if (not os.path.exists(run_dir)):
            try:
                os.mkdir(run_dir)
            except:
                print("%s exists after all" % run_dir)

#        tsdict = dict({"URef": ws, "AnalysisTime":tmax, "UsableTime":tmax}.items() + case.items())
        tsdict = dict({"URef": ws,  "UsableTime":tmax}.items() + case.items())
#        tsoutname = self.ts_file.replace("inp", "wnd")  #self.rawts.ts_file.replace("inp", "wnd")
        # tsoutname = self.ts_file.replace("inp", "bts")  #self.rawts.ts_file.replace("inp", "wnd")
        tsoutname = case_name + '.bts'
        tsoutname = os.path.join(run_dir, tsoutname)
        tssumname = case_name + '.sum'
        # tssumname = self.ts_file.replace("inp", "sum")
        # print("running TurbSim in dir for case:" , run_dir, case, tsdict)

        if self.overwrite or (not self.overwrite and not os.path.exists(tsoutname)):

            tsinput = ptsin.read(os.path.join(self.ts_dir, self.ts_file))
            for key in tsdict:
                tsinput[key] = tsdict[key]
            tsr = ptsm.cfg2tsrun(tsinput)

            ### the random seed:
            ### bug somewhere in complicated pyts use of numpy (only when called inside multiprocessing)
            ### Success via cutting out the middle man!:
            if rs is None:
                tsr.randgen.seed(tsr.RandSeed)
            else:
                tsr.randgen.seed(rs)  ## this does nothing!
                np.random.seed(rs)  ### this does the trick!

            ###tsr = self.add_phase_dist(tsr, rho, tmax)
            tsr.cohere = pyts.cohereModels.main.nwtc()
            tsr.stress = pyts.stressModels.main.uniform(0,0,0)
            tsr.phase = Rinker(rho, np.pi)
            cg = tsr.grid
            tsr.grid = tsGrid(center=cg.center, ny=cg.n_y, nz=cg.n_z,
                         height=cg.height, width=cg.width,
                         time_sec=tmax, dt=cg.dt)

            tsdata = tsr()  ## actually runs turbsim
            ptsm.write(tsdata, tsinput, fname=tsoutname)

        # here we provide the means to link turbsim to fast:
        self.tswind_file = tsoutname
        self.tswind_dir = run_dir

if __name__ == "__main__":

    case_inputs = {}
    case_inputs[("TMax")] = {'vals':[10.], 'group':0}
    case_inputs[("Vhub")] = {'vals':[10., 11., 12.], 'group':1}
    case_inputs[("Rho")] = {'vals':[.2, .25, .3], 'group':1}
    case_inputs[("RandSeed1")] = {'vals':[123,234], 'group':2}
    case_list, case_names = CaseGen_General(case_inputs, dir_matrix='', namebase='pyTurbsim_testing')

    filedict = {
        'ts_dir':"/Users/pgraf/work/wese/templates/turbsim/xloads/",
        'ts_file':"TurbSim.inp",
        'run_dir':"test_run_dir"}

    for idx in range(len(case_list)):
        case = case_list[idx]
        case_name = case_names[idx]
        pyturb = pyTurbsim_wrapper(filedict, case, case_name) # initialize runner with case variable inputs
        pyturb.ny = 20 # example of changing an attribute
        pyturb.execute() # run

        case['tswind_file'] = pyturb.tswind_file
        case['tswind_dir'] = pyturb.tswind_dir
        print("SUCCESS ")
        print("   ", case)



""" testing utilities should be moved to CommonSE 
"""

import numpy as np
from openmdao.api import IndepVarComp, Component, Problem, Group, SqliteRecorder, BaseRecorder



##### Test Setup Utilities #####
def init_IndepVar_add(prob, data):
    """ Add component independent vars from a dictionary of inputs """
    for var in data.keys():
        if type(data[var]) is np.ndarray or type(data[var]) is np.array:
            print var
            prob.root.add(var, IndepVarComp(var, np.zeros_like(data[var])), promotes=['*'])
        else:
            prob.root.add(var, IndepVarComp(var, data[var]), promotes=['*'])
    return prob

def init_IndepVar_set(prob, data):
    """ Set component independent vars from a dictionary of inputs """
    for var in data.keys():
        prob[var] = data[var]
    return prob




##### Check Gradients ##### <- from test_rotor_aeropower_gradients.py
def check_gradient_unit_test(prob, fd='central', step_size=1e-6, tol=1e-6, display=False,
        show_missing_warnings=True, show_scaling_warnings=False, min_grad=1e-6, max_grad=1e6, comp=None):
    """compare provided analytic gradients to finite-difference gradients with unit testing.
    Same as check_gradient, but provides a unit test for each gradient for convenience.
    the unit tests checks that the error for each gradient is less than tol.
    Parameters
    ----------
    comp : obj
        An OpenMDAO component that provides analytic gradients through provideJ()
    fd : str
        the type of finite difference to use.  options are central or forward
    step_size : float
        step size to use in finite differencing
    tol : float
        tolerance for how close the gradients should agree to
    display : boolean
        if True, display results for each gradient
    show_missing_warnings: boolean
        if True, warn for gradients that were not provided
        (they may be ones that are unecessary, but may be ones that were accidentally skipped)
    show_scaling_warnings: boolean
        if True, warn for gradients that are either very small or very large which may lead
        to challenges in solving the full linear system
    min_grad/max_grad : float
        quantifies what "very small" or "very large" means when using show_scaling_warnings
    """

    J_fd, J_fwd, J_rev = check_gradient(prob, fd, step_size, tol, display, show_missing_warnings,
        show_scaling_warnings, min_grad, max_grad)
    if comp == None:
        comp = prob.root.comp
    if "list_deriv_vars" in dir(comp): #  callable(getattr(comp, 'list_deriv_vars')):
        inputs, outputs = comp.list_deriv_vars()
        for output in outputs:
            for input in inputs:
                J = J_fwd[output, input]
                JFD = J_fd[output, input]
                m, n = J.shape
                # print '---------', output, input, '---------'
                # print 'J', J
                # print 'JFD', JFD
                for i in range(m):
                    for j in range(n):
                        if np.abs(J[i, j]) <= tol:
                            errortype = 'absolute'
                            error = J[i, j] - JFD[i, j]
                        else:
                            errortype = 'relative'
                            error = 1.0 - JFD[i, j]/J[i, j]
                        error = np.abs(error)

                        # # display
                        # if error > tol:
                        #     star = ' ***** '
                        # else:
                        #     star = ''
                        #
                        # if display:
                        #     output = '{}{:<20} ({}) {}: ({}, {})'.format(star, error, errortype, name, J[i, j], JFD[i, j])
                        #     print output
                        #
                        # if show_scaling_warnings and J[i, j] != 0 and np.abs(J[i, j]) < min_grad:
                        #     print '*** Warning: The following analytic gradient is very small and may need to be scaled:'
                        #     print '\t(' + comp.__class__.__name__ + ') ' + name + ':', J[i, j]
                        #
                        # if show_scaling_warnings and np.abs(J[i, j]) > max_grad:
                        #     print '*** Warning: The following analytic gradient is very large and may need to be scaled:'
                        #     print '\t(' + comp.__class__.__name__ + ') ' + name + ':', J[i, j]
                        #

                        try:
                            # unittest.assertLessEqual(error, tol)
                            assert error <= tol
                        except AssertionError, e:
                            print '*** error in:', "\n\tOutput: ", output, "\n\tInput: ", input, "\n\tPosition: ", i, j
                            print JFD[i, j], J[i, j]
                            print error, tol
                            raise e
    else:
        for key, value in J_fd.iteritems():
                J = J_fwd[key]
                JFD = J_fd[key]

                m, n = J.shape
                for i in range(m):
                    for j in range(n):
                        if np.abs(J[i, j]) <= tol:
                            errortype = 'absolute'
                            error = J[i, j] - JFD[i, j]
                        else:
                            errortype = 'relative'
                            error = 1.0 - JFD[i, j]/J[i, j]
                        error = np.abs(error)
                        try:
                            # unittest.assertLessEqual(error, tol)
                            assert error <= tol
                        except AssertionError, e:
                            print '*** error in:', "\n\tKey: ", key, "\n\tPosition: ", i, j
                            raise e


def check_gradient(prob, fd='central', step_size=1e-6, tol=1e-6, display=False,
        show_missing_warnings=True, show_scaling_warnings=False, min_grad=1e-6, max_grad=1e6):
    """compare provided analytic gradients to finite-difference gradients
    Parameters
    ----------
    comp : obj
        An OpenMDAO component that provides analytic gradients through provideJ()
    fd : str
        the type of finite difference to use.  options are central or forward
    step_size : float
        step size to use in finite differencing
    tol : float
        tolerance for how close the gradients should agree to
    display : boolean
        if True, display results for each gradient
    show_missing_warnings: boolean
        if True, warn for gradients that were not provided
        (they may be ones that are unecessary, but may be ones that were accidentally skipped)
    show_scaling_warnings: boolean
        if True, warn for gradients that are either very small or very large which may lead
        to challenges in solving the full linear system
    min_grad/max_grad : float
        quantifies what "very small" or "very large" means when using show_scaling_warnings
    Returns
    -------
    names : array(str)
        list of the names of all the gradients
    errorvec : array(float)
        list of all the errors for the gradients.  If the magnitude of the gradient is less than
        tol, then an absolute error is used, otherwise a relative error is used.
    """
    # inputs = comp.list_deriv_vars
    # inputs, outputs = comp.list_deriv_vars()

    # show_missing_warnings = False

    # if show_missing_warnings:
    #     all_inputs = _explodeall(comp, vtype='inputs')
    #     all_outputs = _explodeall(comp, vtype='outputs')
    #     reserved_inputs = ['missing_deriv_policy', 'directory', 'force_fd', 'force_execute', 'eval_only']
    #     reserved_outputs = ['derivative_exec_count', 'itername', 'exec_count']
    #     potential_missed_inputs = list(set(all_inputs) - set(reserved_inputs) - set(inputs))
    #     potential_missed_outputs = list(set(all_outputs) - set(reserved_outputs) - set(outputs))
    #
    #     if len(potential_missed_inputs) > 0 or len(potential_missed_outputs) > 0:
    #         print
    #         print '*** Warning: ' + comp.__class__.__name__ + ' does not supply derivatives for the following'
    #         print '\tinputs:', potential_missed_inputs
    #         print '\toutputs:', potential_missed_outputs
    #         print

    # prob = Problem()
    # prob.root = Group()
    # prob.root.add('comp', comp, promotes=['*'])
    # prob.setup()
    #
    # for i in range(len(inputs)):
    #     prob[inputs[i]] = comp

    prob.run()
    root = prob.root

    # Linearize the model
    root._sys_linearize(root.params, root.unknowns, root.resids)

    data = {}

    # Derivatives should just be checked without parallel adjoint for now.
    voi = None

    jac_fwd = {}
    jac_rev = {}
    jac_fd = {}

    # Check derivative calculations for all comps at every level of the
    # system hierarchy.
    for comp in root.components(recurse=True):
        cname = comp.pathname

        # No need to check comps that don't have any derivs.
        if comp.deriv_options['type'] == 'fd':
            continue

        # IndepVarComps are just clutter too.
        if isinstance(comp, IndepVarComp):
            continue

        data[cname] = {}
        jac_fwd = {}
        jac_rev = {}
        jac_fd = {}

        # try:
        #     params, unknowns = comp.list_deriv_vars()
        # except:
        #     pass
        params = comp.params
        unknowns = comp.unknowns
        resids = comp.resids
        dparams = comp.dpmat[voi]
        dunknowns = comp.dumat[voi]
        dresids = comp.drmat[voi]

        # Skip if all of our inputs are unconnected.
        # if len(dparams) == 0:
        #     continue

        # if out_stream is not None:
        #     out_stream.write('-'*(len(cname)+15) + '\n')
        #     out_stream.write("Component: '%s'\n" % cname)
        #     out_stream.write('-'*(len(cname)+15) + '\n')

        states = comp.states

        param_list = [item for item in dparams if not \
                      dparams.metadata(item).get('pass_by_obj')]
        param_list.extend(states)

        # Create all our keys and allocate Jacs
        for p_name in param_list:

            dinputs = dunknowns if p_name in states else dparams
            p_size = np.size(dinputs[p_name])

            # Check dimensions of user-supplied Jacobian
            for u_name in unknowns:

                u_size = np.size(dunknowns[u_name])
                if comp._jacobian_cache:

                    # We can perform some additional helpful checks.
                    if (u_name, p_name) in comp._jacobian_cache:

                        user = comp._jacobian_cache[(u_name, p_name)].shape

                        # User may use floats for scalar jacobians
                        if len(user) < 2:
                            user = (user[0], 1)

                        if user[0] != u_size or user[1] != p_size:
                            msg = "derivative in component '{}' of '{}' wrt '{}' is the wrong size. " + \
                                  "It should be {}, but got {}"
                            msg = msg.format(cname, u_name, p_name, (u_size, p_size), user)
                            raise ValueError(msg)

                jac_fwd[(u_name, p_name)] = np.zeros((u_size, p_size))
                jac_rev[(u_name, p_name)] = np.zeros((u_size, p_size))

        # Reverse derivatives first
        for u_name in dresids:
            u_size = np.size(dunknowns[u_name])

            # Send columns of identity
            for idx in range(u_size):
                dresids.vec[:] = 0.0
                root.clear_dparams()
                dunknowns.vec[:] = 0.0

                dresids._dat[u_name].val[idx] = 1.0
                try:
                    comp.apply_linear(params, unknowns, dparams,
                                      dunknowns, dresids, 'rev')
                finally:
                    dparams._apply_unit_derivatives()

                for p_name in param_list:

                    dinputs = dunknowns if p_name in states else dparams
                    # try:
                    jac_rev[(u_name, p_name)][idx, :] = dinputs._dat[p_name].val
                    # except:
                    #     pass
        # Forward derivatives second
        for p_name in param_list:

            dinputs = dunknowns if p_name in states else dparams
            p_size = np.size(dinputs[p_name])

            # Send columns of identity
            for idx in range(p_size):
                dresids.vec[:] = 0.0
                root.clear_dparams()
                dunknowns.vec[:] = 0.0

                dinputs._dat[p_name].val[idx] = 1.0
                dparams._apply_unit_derivatives()
                comp.apply_linear(params, unknowns, dparams,
                                  dunknowns, dresids, 'fwd')

                for u_name, u_val in dresids.vec_val_iter():
                    jac_fwd[(u_name, p_name)][:, idx] = u_val

        # Finite Difference goes last
        dresids.vec[:] = 0.0
        root.clear_dparams()
        dunknowns.vec[:] = 0.0

        # Component can request to use complex step.
        if comp.deriv_options['form'] == 'complex_step':
            fd_func = comp.complex_step_jacobian
        else:
            fd_func = comp.fd_jacobian
        jac_fd = fd_func(params, unknowns, resids, option_overrides=comp.deriv_options) #EMG: derv_options were not being passed

        # # Assemble and Return all metrics.
        # _assemble_deriv_data(chain(dparams, states), resids, data[cname],
        #                      jac_fwd, jac_rev, jac_fd, out_stream,
        #                      c_name=cname)

    return jac_fd, jac_fwd, jac_rev


import unittest

from wisdem.test.test_turbinecostsse import test_turbine_costsse_2015


def suite():
    suite = unittest.TestSuite( (test_turbine_costsse_2015.suite(),
    ) )
    return suite


if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())
        



import os
import xlrd

""" 
Script to generate the Output channel dictionaries.  Included so they can be easily updated
for future FAST versions and additional FAST modules.

by Evan Gaertner
"""


def GetOutlistParameters(fname_vars_out, xl_files_in, sheet_list, write_mode, final_dict):

    f = open(fname_vars_out, write_mode)
    f.write('""" Generated from FAST OutListParameters.xlsx files with AeroelasticSE/src/AeroelasticSE/Util/create_output_vars.py """\n')

    for fname, sheet_name in zip(xl_files_in, sheet_list):
        book = xlrd.open_workbook(fname)
        sheet_idx = book.sheet_names().index(sheet_name)
        sheet = book.sheet_by_index(sheet_idx)

        print(sheet_name, '\t:\t', fname)

        # get data from sheet
        outlist = {}
        headers = {}
        var_names = []
        n = sheet.nrows
        n_dup = 0

        for idx in range(1, sheet.nrows):
            row = sheet.row_values(idx)
            if row[1]!='' and row[5]!='':
                outvar = str(row[1])
                var_names.append(str(row[1]))
                outlist[outvar] = {}
                outlist[outvar]['desc'] = ''.join([i if ord(i) < 128 else ' ' for i in row[3]])
                outlist[outvar]['conv'] = ''.join([i if ord(i) < 128 else ' ' for i in row[4]])
                outlist[outvar]['unit'] = ''.join([i if ord(i) < 128 else ' ' for i in row[5]])
                outlist[outvar]['idx'] = idx

                for var in row[2].split(','):
                    if var.strip() != '':
                        outlist[str(var.strip())] = outlist[outvar]
                        var_names.append(str(var.strip()))
                        n += 1
                        n_dup +=1

            else:
                headers[idx+n_dup] = str(row[0])
        
        # write out variables + metadata
        f.write('\n\n""" ' + sheet_name + ' """\n')
        f.write(sheet_name + ' = {}\n')

        var_count = 0
        for idx in range(1, n):
            if idx in headers.keys():
                f.write('\n# ' + headers[idx] + '\n')
            else:
                var = var_names[var_count]
                f.write(("%s['%s']"%(sheet_name, var)).ljust(30) + "= False     # %s; %s; %s\n"%(outlist[var]['unit'], outlist[var]['desc'], outlist[var]['conv']))
                var_count += 1

    # combine in one dict
    f.write('\n\n""" Final Output Dictionary """\n')
    f.write('%s = {}\n'%final_dict)
    for var in sheet_list:
        f.write(("%s['%s']"%(final_dict, var)).ljust(30) + "= %s\n"%var)


    f.close()


if __name__=="__main__":

    # # File Locations
    # openfast_dir = 'C:/Users/egaertne/WT_Codes/openfast'
    # outlist_fast_lib = os.path.join(openfast_dir, 'modules-local', 'fast-library', 'src' , 'OutListParameters.xlsx')
    # outlist_inflow = os.path.join(openfast_dir, 'modules-local', 'inflowwind', 'src' , 'OutListParameters.xlsx')

    # # Sheets to grab
    # sheet_list_fast_lib = ['ElastoDyn', 'BeamDyn', 'ServoDyn', 'AeroDyn']
    # xl_files_fast_lib = [outlist_fast_lib]*len(sheet_list_fast_lib)
    # sheet_list_inflow = ['InflowWind']
    # xl_files_inflow = [outlist_inflow]*len(sheet_list_inflow)
    # xl_files_in = xl_files_fast_lib + xl_files_inflow
    # sheet_list = sheet_list_fast_lib + sheet_list_inflow

    # # Output naming
    # fname_vars_out = 'FAST_vars_out.py'

    # # Run
    # GetOutlistParameters(fname_vars_out, xl_files_in, sheet_list, 'w', 'FstOutput')


    # # Add FAST7
    # xl_files_in_FAST7 = ['C:/Users/egaertne/WT_Codes/FAST_v7.02.00d-bjj/OutListParameters.xlsx']
    # sheet_list_FAST7 = ['OutList']
    # GetOutlistParameters(fname_vars_out, xl_files_in_FAST7, sheet_list_FAST7, 'a', 'Fst7Output')

    ### adding hydro ####################

    # File Locations
    xl_files_in = ["C:/Users/egaertne/Documents/Projects/AeroelasticSE/Hydro_Outlist/Waves2OutListParameters.xlsx",
                   "C:/Users/egaertne/Documents/Projects/AeroelasticSE/Hydro_Outlist/HydroDynOutListParameters.xlsx",
                   "C:/Users/egaertne/Documents/Projects/AeroelasticSE/Hydro_Outlist/MorisonOutListParameters.xlsx",
                   "C:/Users/egaertne/Documents/Projects/AeroelasticSE/Hydro_Outlist/WAMIT2OutListParameters.xlsx",
                   "C:/Users/egaertne/Documents/Projects/AeroelasticSE/Hydro_Outlist/WAMITOutListParameters.xlsx",
                   "C:/Users/egaertne/Documents/Projects/AeroelasticSE/Hydro_Outlist/SubDynOutListParameters.xlsx"]
    
    # Sheets to grab
    sheet_list = ['WAMIT', 'HydroDyn', 'Morison', 'WAMIT', 'WAMIT', 'SubDyn']

    # Output naming
    fname_vars_out = 'FAST_vars_out.py'

    # Run
    GetOutlistParameters(fname_vars_out, xl_files_in, sheet_list, 'w', 'FstOutput')



from openmdao.api import Problem
from wisdem.turbine_costsse.turbine_costsse_2015 import Turbine_CostsSE_2015
import unittest
import numpy as np

class TestNewAssembly(unittest.TestCase):

  def setUp(self):
    turbine = Turbine_CostsSE_2015(verbosity=False)
    self.prob = Problem(turbine)
    self.prob.setup()

    self.prob['blade_mass']          = 17650.67  # inline with the windpact estimates
    self.prob['hub_mass']            = 31644.5
    self.prob['pitch_system_mass']   = 17004.0
    self.prob['spinner_mass']        = 1810.5
    self.prob['lss_mass']            = 31257.3
    #bearingsMass'] = 9731.41
    self.prob['main_bearing_mass']   = 9731.41 / 2
    self.prob['gearbox_mass']        = 30237.60
    self.prob['hss_mass']            = 1492.45
    self.prob['generator_mass']      = 16699.85
    self.prob['bedplate_mass']       = 93090.6
    self.prob['yaw_mass']            = 11878.24
    self.prob['tower_mass']          = 434559.0
    self.prob['vs_electronics_mass'] = 1000.
    self.prob['hvac_mass']           = 1000.
    self.prob['cover_mass']          = 1000.
    self.prob['platforms_mass']      = 1000.
    self.prob['transformer_mass']    = 1000.

    # other inputs
    self.prob['machine_rating']      = 5000.0
    self.prob['blade_number']        = 3
    self.prob['crane']               = True
    self.prob['main_bearing_number'] = 2


  def test1(self):

    self.prob.run_model()

    self.assertEqual(np.round(self.prob['rotor_cost'], 2), 1292397.85)
    self.assertEqual(np.round(self.prob['generator_cost'], 2), 207078.14)
    self.assertEqual(np.round(self.prob['transformer_cost'], 2), 18800.00)
    self.assertEqual(np.round(self.prob['turbine_cost'], 2), 4448107.48)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestNewAssembly))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())




"""eesgOM.py
Created by Latha Sethuraman, Katherine Dykes.
Copyright (c) NREL. All rights reserved.
Electromagnetic design based on conventional magnetic circuit laws
Structural design based on McDonald's thesis 

The OpenMDAO part of an electrically excited generator module.
The pure-python part is in eesgSE.py
"""

from openmdao.api import Group, Problem, ExplicitComponent,ExecComp,IndepVarComp,ScipyOptimizeDriver
import numpy as np
from math import pi, cos, sqrt, radians, sin, exp, log10, log, tan, atan
import sys, os
from eesgSE import EESG

class EESG_OM(ExplicitComponent):
    """ Estimates overall mass dimensions and Efficiency of Electrically Excited Synchronous generator. """
    
    def setup(self):
        # EESG generator design inputs
        #self.add_input('r_s', val=0.0, units ='m', desc='airgap radius r_s')
        self.add_input('rad_ag', val=0.0, units ='m', desc='airgap radius')
        self.add_input('len_s',  val=0.0, units ='m', desc='Stator core length l_s')
        self.add_input('h_s',    val=0.0, units ='m', desc='Yoke height h_s')
        self.add_input('tau_p',  val=0.0, units ='m', desc='Pole pitch self.tau_p')
        
        self.add_input('machine_rating', val=0.0, units ='W',   desc='Machine rating')
        self.add_input('n_nom',          val=0.0, units ='rpm', desc='rated speed')
        self.add_input('Torque',         val=0.0, units ='N*m', desc='Rated torque ')
        self.add_input('I_f',            val=0.0, units='A',    desc='Excitation current')
        self.add_input('N_f',            val=0.0, units='A',    desc='field turns')
        self.add_input('h_ys',           val=0.0, units ='m',   desc='Yoke height')
        self.add_input('h_yr',           val=0.0, units ='m',   desc='rotor yoke height')
        
        # structural design variables
        self.add_input('n_s',  val=0.0,            desc='number of stator arms n_s')
        self.add_input('b_st', val=0.0, units='m', desc='arm width b_st')
        self.add_input('d_s',  val=0.0, units='m', desc='arm depth d_s')
        self.add_input('t_ws', val=0.0, units='m', desc='arm depth thickness self.t_ws')
        self.add_input('n_r',  val=0.0,            desc='number of arms n')
        self.add_input('b_r',  val=0.0, units='m', desc='arm width b_r')
        self.add_input('d_r',  val=0.0, units='m', desc='arm depth d_r')
        self.add_input('t_wr', val=0.0, units='m', desc='arm depth thickness self.t_wr')
        self.add_input('R_o',  val=0.0, units='m', desc='Shaft radius')
        
        # EESG generator design outputs
        
        # Magnetic loading
        self.add_output('B_symax', val=0.0, units='T', desc='Peak Stator Yoke flux density B_ymax')
        self.add_output('B_tmax',  val=0.0, units='T', desc='Peak Teeth flux density')
        self.add_output('B_rymax', val=0.0, units='T', desc='Peak Rotor yoke flux density')
        self.add_output('B_gfm',   val=0.0, units='T', desc='Average air gap flux density B_g')
        self.add_output('B_g',     val=0.0, units='T', desc='Peak air gap flux density B_g')
        self.add_output('B_pc',    val=0.0, units='T', desc='Pole core flux density')
        
        # Stator design
        self.add_output('N_s',       val=0.0,                desc='Number of turns in the stator winding')
        self.add_output('b_s',       val=0.0, units ='m',    desc='slot width')
        self.add_output('b_t',       val=0.0, units ='m',    desc='tooth width')
        self.add_output('A_Cuscalc', val=0.0, units='mm**2', desc='Stator Conductor cross-section mm^2')
        self.add_output('S',         val=0.0,                desc='Stator slots')
        
        # # Output parameters : Rotor design
        self.add_output('h_p',       val=0.0, units ='m',    desc='Pole height')
        self.add_output('b_p',       val=0.0, units ='m',    desc='Pole width')
        self.add_output('p',         val=0.0,                desc='No of pole pairs')
        self.add_output('n_brushes', val=0.0,                desc='number of brushes')
        self.add_output('A_Curcalc', val=0.0, units='mm**2', desc='Rotor Conductor cross-section')
        
        # Output parameters : Electrical performance
        self.add_output('E_s',            val=0.0, units='V',       desc='Stator phase voltage')
        self.add_output('f',              val=0.0,                  desc='Generator output frequency')
        self.add_output('I_s',            val=0.0, units='A',       desc='Generator output phase current')
        self.add_output('R_s',            val=0.0, units='ohm',     desc='Stator resistance')
        self.add_output('R_r',            val=0.0, units='ohm',     desc='Rotor resistance')
        self.add_output('L_m',            val=0.0, units='H',       desc='Stator synchronising inductance')
        self.add_output('J_s',            val=0.0, units='A*m**-2', desc='Stator Current density')
        self.add_output('J_f',            val=0.0, units='A*m**-2', desc='rotor Current density')
        self.add_output('A_1',            val=0.0,                  desc='Specific current loading')
        self.add_output('Load_mmf_ratio', val=0.0,                  desc='mmf_ratio')
        
        # Objective functions and output
        self.add_output('Mass',    val=0.0, units='kg',  desc='Actual mass')
        self.add_output('K_rad',   val=0.0,              desc='K_rad')
        self.add_output('Losses',  val=0.0, units='W',   desc='Total loss')
        self.add_output('gen_eff', val=0.0,              desc='Generator efficiency')   # units='pct'
        
        # Structural performance
        self.add_output('u_Ar',    val=0.0, units='m',    desc='Rotor radial deflection')
        self.add_output('y_Ar',    val=0.0, units='m',    desc='Rotor axial deflection')
        self.add_output('z_A_r',   val=0.0, units='m',    desc='Rotor circumferential deflection')
        self.add_output('u_As',    val=0.0, units='m',    desc='Stator radial deflection')
        self.add_output('y_As',    val=0.0, units='m',    desc='Stator axial deflection')
        self.add_output('z_A_s',   val=0.0, units='m',    desc='Stator circumferential deflection')  
        self.add_output('u_all_r', val=0.0, units='m',    desc='Allowable radial rotor')
        self.add_output('u_all_s', val=0.0, units='m',    desc='Allowable radial stator')
        self.add_output('y_all',   val=0.0, units='m',    desc='Allowable axial')
        self.add_output('z_all_s', val=0.0, units='m',    desc='Allowable circum stator')
        self.add_output('z_all_r', val=0.0, units='m',    desc='Allowable circum rotor')
        self.add_output('b_all_s', val=0.0, units='m',    desc='Allowable arm')
        self.add_output('b_all_r', val=0.0, units='m',    desc='Allowable arm dimensions')
        self.add_output('TC1',     val=0.0, units='m**3', desc='Torque constraint')
        self.add_output('TC2',     val=0.0, units='m**3', desc='Torque constraint-rotor')
        self.add_output('TC3',     val=0.0, units='m**3', desc='Torque constraint-stator')
        
        # Material properties
        self.add_input('rho_Fes',    val=0.0, units='kg*m**-3', desc='Structural Steel density ')
        self.add_input('rho_Fe',     val=0.0, units='kg*m**-3', desc='Magnetic Steel density ')
        self.add_input('rho_Copper', val=0.0, units='kg*m**-3', desc='Copper density ')
        
        # Mass Outputs
        self.add_output('Copper',          val=0.0, units='kg', desc='Copper Mass')
        self.add_output('Iron',            val=0.0, units='kg', desc='Electrical Steel Mass')
        self.add_output('Structural_mass', val=0.0, units='kg', desc='Structural Mass')
        
        # Other parameters
        self.add_output('Power_ratio',       val=0.0,             desc='Power_ratio')
        self.add_output('Slot_aspect_ratio', val=0.0,             desc='Stator slot aspect ratio')
        self.add_output('R_out',             val=0.0, units ='m', desc='Outer radius')
        
        # inputs/outputs for interface with drivese
        self.add_input('shaft_cm',     val=np.zeros(3), units='m',       desc='Main Shaft CM')
        self.add_input('shaft_length', val=0.0,         units='m',       desc='main shaft length')
        self.add_output('I',           val=np.zeros(3), units='kg*m**2', desc='Moments of Inertia for the component [Ixx, Iyy, Izz] around its center of mass')
        self.add_output('cm',          val=np.zeros(3), units='m',       desc='COM [x,y,z]')

        self.declare_partials('*', '*', method='fd', form='central', step=1e-6)

        
    def compute(self, inputs, outputs):
        # Unpack outputs
        rad_ag            = inputs['rad_ag']
        len_s             = inputs['len_s']
        h_s               = inputs['h_s']
        tau_p             = inputs['tau_p']
        N_f               = inputs['N_f']
        I_f               = inputs['I_f']
        h_ys              = inputs['h_ys']
        h_yr              = inputs['h_yr']
        machine_rating    = inputs['machine_rating']
        n_nom             = inputs['n_nom']
        Torque            = inputs['Torque']
        
        b_st              = inputs['b_st']
        d_s               = inputs['d_s']
        t_ws              = inputs['t_ws']
        n_r               = inputs['n_r']
        n_s               = inputs['n_s']
        b_r               = inputs['b_r']
        d_r               = inputs['d_r']
        t_wr              = inputs['t_wr']
        
        R_o               = inputs['R_o']
        rho_Fe            = inputs['rho_Fe']
        rho_Copper        = inputs['rho_Copper']
        rho_Fes           = inputs['rho_Fes']
        shaft_cm          = inputs['shaft_cm']
        shaft_length      = inputs['shaft_length']
        
        eesg = EESG()
        
        (outputs['B_symax'], outputs['B_tmax'], outputs['B_rymax'], outputs['B_gfm'], outputs['B_g'], outputs['B_pc'], 
        outputs['N_s'], outputs['b_s'], outputs['b_t'], outputs['A_Cuscalc'], outputs['A_Curcalc'], outputs['b_p'], 
        outputs['h_p'], outputs['p'], outputs['E_s'], outputs['f'], outputs['I_s'], outputs['R_s'], outputs['L_m'], 
        outputs['A_1'], outputs['J_s'], outputs['R_r'], outputs['Losses'], outputs['Load_mmf_ratio'], outputs['Power_ratio'], 
        outputs['n_brushes'], outputs['J_f'], outputs['K_rad'], outputs['gen_eff'], outputs['S'], 
        outputs['Slot_aspect_ratio'], outputs['Copper'], outputs['Iron'], outputs['u_Ar'], outputs['y_Ar'], 
        outputs['z_A_r'], outputs['u_As'], outputs['y_As'], outputs['z_A_s'], outputs['u_all_r'], outputs['u_all_s'], 
        outputs['y_all'], outputs['z_all_s'], outputs['z_all_r'], outputs['b_all_s'], outputs['b_all_r'], outputs['TC1'], 
        outputs['TC2'], outputs['TC3'], outputs['R_out'], outputs['Structural_mass'], outputs['Mass'], outputs['cm'], outputs['I']) \
         = eesg.compute(rad_ag, len_s, h_s, tau_p, N_f, I_f, h_ys, h_yr, machine_rating, n_nom, Torque,               
                    b_st, d_s, t_ws, n_r, n_s, b_r, d_r, t_wr, R_o, rho_Fe, rho_Copper, rho_Fes, shaft_cm, shaft_length)
        
'''
        outputs['B_symax']           = B_symax
        outputs['B_tmax']            = B_tmax
        outputs['B_rymax']           = B_rymax
        outputs['B_gfm']             = B_gfm
        outputs['B_g']               = B_g
        outputs['B_pc']              = B_pc
        outputs['N_s']               = N_s
        outputs['b_s']               = b_s

        outputs['b_t']               = b_t
        outputs['A_Cuscalc']         = A_Cuscalc
        outputs['A_Curcalc']         = A_Curcalc
        outputs['b_p']               = b_p
        outputs['h_p']               = h_p
        outputs['p']                 = p
        outputs['E_s']               = E_s
        outputs['f']                 = f

        outputs['I_s']               = I_s
        outputs['R_s']               = R_s
        outputs['L_m']               = L_m
        outputs['A_1']               = A_1
        outputs['J_s']               = J_s
        outputs['R_r']               = R_r
        outputs['Losses']            = Losses

        outputs['Load_mmf_ratio']    = Load_mmf_ratio
        outputs['Power_ratio']       = Power_ratio
        outputs['n_brushes']         = n_brushes
        outputs['J_f']               = J_f
        outputs['K_rad']             = K_rad
        outputs['gen_eff']           = gen_eff
        outputs['S']                 = S

        outputs['Slot_aspect_ratio'] = Slot_aspect_ratio
        outputs['Copper']            = Copper
        outputs['Iron']              = Iron
        outputs['u_Ar']              = u_Ar
        outputs['y_Ar']              = y_Ar

        outputs['z_A_r']             = z_A_r
        outputs['u_As']              = u_As
        outputs['y_As']              = y_As
        outputs['z_A_s']             = z_A_s
        outputs['u_all_r']           = u_all_r
        outputs['u_all_s']           = u_all_s

        outputs['y_all']             = y_all
        outputs['z_all_s']           = z_all_s
        outputs['z_all_r']           = z_all_r
        outputs['b_all_s']           = b_all_s
        outputs['b_all_r']           = b_all_r
        outputs['TC1']               = TC1

        outputs['TC2']               = TC2
        outputs['TC3']               = TC3
        outputs['R_out']             = R_out
        outputs['Structural_mass']   = Structural_mass
        outputs['Mass']              = Mass
        outputs['cm']                = cm
        outputs['I']                 = I
        
B_symax, B_tmax, B_rymax, B_gfm, B_g, B_pc, N_s, b_s, b_t, A_Cuscalc, A_Curcalc, b_p, 
h_p, p, E_s, f, I_s, R_s, L_m, A_1, J_s, R_r, Losses, Load_mmf_ratio, Power_ratio, 
n_brushes, J_f, K_rad, gen_eff, S, Slot_aspect_ratio, Copper, Iron, u_Ar, y_Ar, 
z_A_r, u_As, y_As, z_A_s, u_all_r, u_all_s, y_all, z_all_s, z_all_r, b_all_s, b_all_r, TC1, 
TC2, TC3, R_out, Structural_mass, Mass, cm, I
'''



import numpy as np
import numpy.testing as npt
import unittest
import wisdem.towerse.tower as tow
import openmdao.api as om
from wisdem.commonse import gravity as g
import copy

class TestTowerSE(unittest.TestCase):
    def setUp(self):
        self.inputs = {}
        self.outputs = {}
        self.discrete_inputs = {}
        self.discrete_outputs = {}

        
    def testMonopileFoundation(self):
        # Test Land
        self.inputs['tower_section_height'] = 2.*np.ones(2)
        self.inputs['tower_outer_diameter'] = 3.*np.ones(3)
        self.inputs['tower_wall_thickness'] = 0.1*np.ones(2)
        self.inputs['suctionpile_depth'] = 0.0
        self.inputs['suctionpile_depth_diam_ratio'] = 0.0
        self.inputs['foundation_height'] = 0.0
        myobj = tow.MonopileFoundation(nPoints=3, monopile=False)
        myobj.compute(self.inputs, self.outputs)
        npt.assert_equal(self.outputs['section_height_out'], self.inputs['tower_section_height'])
        npt.assert_equal(self.outputs['outer_diameter_out'], self.inputs['tower_outer_diameter'])
        npt.assert_equal(self.outputs['wall_thickness_out'], self.inputs['tower_wall_thickness'])
        npt.assert_equal(self.outputs['foundation_height_out'], self.inputs['foundation_height'])

        # Test Land with bad suctionpile input
        self.inputs['suctionpile_depth'] = 10.0
        myobj.compute(self.inputs, self.outputs)
        npt.assert_equal(self.outputs['section_height_out'], self.inputs['tower_section_height'])
        npt.assert_equal(self.outputs['outer_diameter_out'], self.inputs['tower_outer_diameter'])
        npt.assert_equal(self.outputs['wall_thickness_out'], self.inputs['tower_wall_thickness'])
        npt.assert_equal(self.outputs['foundation_height_out'], self.inputs['foundation_height'])
        
        # Test monopile with pile
        self.inputs['suctionpile_depth'] = 10.0
        self.inputs['foundation_height'] = -30.0
        myobj = tow.MonopileFoundation(nPoints=3, monopile=True)
        myobj.compute(self.inputs, self.outputs)
        npt.assert_equal(self.outputs['section_height_out'], np.array([10., 2., 2.]))
        npt.assert_equal(self.outputs['outer_diameter_out'], 3.*np.ones(4))
        npt.assert_equal(self.outputs['wall_thickness_out'], 0.1*np.ones(3))
        npt.assert_equal(self.outputs['foundation_height_out'], self.inputs['foundation_height']-self.inputs['suctionpile_depth'])
        
        # Test monopile with gravity
        self.inputs['suctionpile_depth'] = 0.0
        myobj = tow.MonopileFoundation(nPoints=3, monopile=True)
        myobj.compute(self.inputs, self.outputs)
        npt.assert_equal(self.outputs['section_height_out'], np.array([0.1, 2., 2.]))
        npt.assert_equal(self.outputs['outer_diameter_out'], 3.*np.ones(4))
        npt.assert_equal(self.outputs['wall_thickness_out'], 0.1*np.ones(3))
        npt.assert_equal(self.outputs['foundation_height_out'], self.inputs['foundation_height']-0.1)

        # Test monopile with depth ratio
        self.inputs['suctionpile_depth'] = 0.0
        self.inputs['suctionpile_depth_diam_ratio'] = 4.0
        myobj = tow.MonopileFoundation(nPoints=3, monopile=True)
        myobj.compute(self.inputs, self.outputs)
        npt.assert_equal(self.outputs['section_height_out'], np.array([12., 2., 2.]))
        npt.assert_equal(self.outputs['outer_diameter_out'], 3.*np.ones(4))
        npt.assert_equal(self.outputs['wall_thickness_out'], 0.1*np.ones(3))
        npt.assert_equal(self.outputs['foundation_height_out'], self.inputs['foundation_height']-12.)
        
        
    def testTowerDisc(self):
        # Test Land
        self.inputs['hub_height'] = 100.0
        self.inputs['z_param'] = np.array([0., 40., 80.])
        myobj = tow.TowerDiscretization(nPoints=3)
        myobj.compute(self.inputs, self.outputs)
        self.assertEqual(self.outputs['height_constraint'], 20.0)
        
        # Test monopile 
        self.inputs['z_param'] = np.array([-50., -30, 0.0, 40., 80.])
        myobj = tow.TowerDiscretization(nPoints=5)
        myobj.compute(self.inputs, self.outputs)
        self.assertEqual(self.outputs['height_constraint'], 20.0)

        
    def testTowerMass(self):

        self.inputs['z_full'] = np.array([-50., -30, 0.0, 40., 80.])
        self.inputs['cylinder_mass'] = 1e3*np.ones(4)
        self.inputs['cylinder_cost'] = 1e5
        self.inputs['cylinder_center_of_mass'] = 10.0
        self.inputs['cylinder_section_center_of_mass'] = self.inputs['z_full'][:-1] + 0.5*np.diff(self.inputs['z_full'])
        self.inputs['cylinder_I_base'] = 1e4*np.r_[np.ones(3), np.zeros(3)]
        self.inputs['transition_piece_height'] = 20.0
        self.inputs['transition_piece_mass'] = 1e2
        self.inputs['gravity_foundation_mass'] = 1e2
        self.inputs['foundation_height'] = -30.
        
        myobj = tow.TowerMass(nFull=5)
        myobj.compute(self.inputs, self.outputs)
        
        self.assertEqual(self.outputs['tower_raw_cost'], self.inputs['cylinder_cost'])
        npt.assert_equal(self.outputs['tower_I_base'], self.inputs['cylinder_I_base'])
        self.assertEqual(self.outputs['tower_center_of_mass'], (4*1e3*10.0 + 1e2*20.0 + 1e2*-30.0)/(4*1e3+2e2) )
        npt.assert_equal(self.outputs['tower_section_center_of_mass'], self.inputs['cylinder_section_center_of_mass'])
        self.assertEqual(self.outputs['monopile_mass'], 1e3*2.5 + 2*1e2)
        self.assertEqual(self.outputs['monopile_cost'], self.inputs['cylinder_cost']*2.5/4.0)
        self.assertEqual(self.outputs['monopile_length'], 70.0)
        self.assertEqual(self.outputs['tower_mass'], 1e3*(4-2.5))


    def testPreFrame(self):
        
        # Test Land 
        self.inputs['z'] = 10. * np.arange(0,7)
        self.inputs['d'] = 6. * np.ones(self.inputs['z'].shape)
        self.inputs['mass'] = 1e5
        self.inputs['mI']   = np.r_[1e5, 1e5, 2e5, np.zeros(3)]
        self.inputs['mrho'] = np.array([-3., 0.0, 1.0])
        self.inputs['transition_piece_mass'] = 0.0
        self.inputs['transition_piece_height'] = 0.0
        self.inputs['gravity_foundation_mass'] = 0.0
        self.inputs['foundation_height'] = 0.0
        self.inputs['rna_F'] = 1e5*np.array([2., 3., 4.,])
        self.inputs['rna_M'] = 1e6*np.array([2., 3., 4.,])
        self.inputs['k_monopile'] = np.zeros(6)

        myobj = tow.TowerPreFrame(nPoints=3, nFull=7, monopile=False)
        myobj.compute(self.inputs, self.outputs)

        npt.assert_equal(self.outputs['kidx'], np.array([0]))
        npt.assert_equal(self.outputs['kx'], np.array([1e16]))
        npt.assert_equal(self.outputs['ky'], np.array([1e16]))
        npt.assert_equal(self.outputs['kz'], np.array([1e16]))
        npt.assert_equal(self.outputs['ktx'], np.array([1e16]))
        npt.assert_equal(self.outputs['kty'], np.array([1e16]))
        npt.assert_equal(self.outputs['ktz'], np.array([1e16]))

        npt.assert_equal(self.outputs['midx'], np.array([6, 0, 0]))
        npt.assert_equal(self.outputs['m'], np.array([1e5, 0, 0]))
        npt.assert_equal(self.outputs['mrhox'], np.array([-3., 0., 0.]))
        npt.assert_equal(self.outputs['mrhoy'], np.array([0., 0., 0.]))
        npt.assert_equal(self.outputs['mrhoz'], np.array([1., 0., 0.]))
        npt.assert_equal(self.outputs['mIxx'], np.array([1e5, 0., 0.]))
        npt.assert_equal(self.outputs['mIyy'], np.array([1e5, 0., 0.]))
        npt.assert_equal(self.outputs['mIzz'], np.array([2e5, 0., 0.]))
        npt.assert_equal(self.outputs['mIxy'], np.zeros(3))
        npt.assert_equal(self.outputs['mIxz'], np.zeros(3))
        npt.assert_equal(self.outputs['mIyz'], np.zeros(3))

        npt.assert_equal(self.outputs['plidx'], np.array([6]))
        npt.assert_equal(self.outputs['Fx'], np.array([2e5]))
        npt.assert_equal(self.outputs['Fy'], np.array([3e5]))
        npt.assert_equal(self.outputs['Fz'], np.array([4e5]))
        npt.assert_equal(self.outputs['Mxx'], np.array([2e6]))
        npt.assert_equal(self.outputs['Myy'], np.array([3e6]))
        npt.assert_equal(self.outputs['Mzz'], np.array([4e6]))

        # Test Monopile 
        self.inputs['z'] = 10. * np.arange(-6,7)
        self.inputs['d'] = 6. * np.ones(self.inputs['z'].shape)
        self.inputs['transition_piece_mass'] = 1e3
        self.inputs['transition_piece_height'] = 10.0
        self.inputs['gravity_foundation_mass'] = 1e4
        self.inputs['foundation_height'] = -30.0
        self.inputs['rna_F'] = 1e5*np.array([2., 3., 4.,])
        self.inputs['rna_M'] = 1e6*np.array([2., 3., 4.,])
        self.inputs['k_monopile'] = 20. + np.arange(6)

        myobj = tow.TowerPreFrame(nPoints=5, nFull=13, monopile=True)
        myobj.compute(self.inputs, self.outputs)

        npt.assert_equal(self.outputs['kidx'], np.array([0, 1, 2, 3]))
        npt.assert_equal(self.outputs['kx'], 20.*np.ones(4))
        npt.assert_equal(self.outputs['ky'], 22.*np.ones(4))
        npt.assert_equal(self.outputs['kz'], 24.*np.ones(4))
        npt.assert_equal(self.outputs['ktx'], 21.*np.ones(4))
        npt.assert_equal(self.outputs['kty'], 23.*np.ones(4))
        npt.assert_equal(self.outputs['ktz'], 25.*np.ones(4))

        npt.assert_equal(self.outputs['midx'], np.array([12, 7, 0]))
        npt.assert_equal(self.outputs['m'], np.array([1e5, 1e3, 1e4]))
        npt.assert_equal(self.outputs['mrhox'], np.array([-3., 0., 0.]))
        npt.assert_equal(self.outputs['mrhoy'], np.array([0., 0., 0.]))
        npt.assert_equal(self.outputs['mrhoz'], np.array([1., 0., 0.]))
        npt.assert_equal(self.outputs['mIxx'], np.array([1e5, 1e3*9*0.5, 1e4*9*0.25]))
        npt.assert_equal(self.outputs['mIyy'], np.array([1e5, 1e3*9*0.5, 1e4*9*0.25]))
        npt.assert_equal(self.outputs['mIzz'], np.array([2e5, 1e3*9, 1e4*9*0.5]))
        npt.assert_equal(self.outputs['mIxy'], np.zeros(3))
        npt.assert_equal(self.outputs['mIxz'], np.zeros(3))
        npt.assert_equal(self.outputs['mIyz'], np.zeros(3))

        npt.assert_equal(self.outputs['plidx'], np.array([12]))
        npt.assert_equal(self.outputs['Fx'], np.array([2e5]))
        npt.assert_equal(self.outputs['Fy'], np.array([3e5]))
        npt.assert_equal(self.outputs['Fz'], np.array([4e5]))
        npt.assert_equal(self.outputs['Mxx'], np.array([2e6]))
        npt.assert_equal(self.outputs['Myy'], np.array([3e6]))
        npt.assert_equal(self.outputs['Mzz'], np.array([4e6]))


    def testProblemLand(self):
        prob = om.Problem()
        prob.model = tow.TowerSE(nLC=1, nPoints=3, nFull=7, wind='PowerWind', topLevelFlag=True, monopile=False)
        prob.setup()

        prob['shearExp'] = 0.2
        prob['hub_height'] = 80.0
        prob['foundation_height'] = 0.0
        prob['transition_piece_height'] = 0.0
        prob['transition_piece_mass'] = 0.0
        prob['gravity_foundation_mass'] = 0.0
        prob['tower_section_height'] = 40.0*np.ones(2)
        prob['tower_outer_diameter'] = 10.0*np.ones(3)
        prob['tower_wall_thickness'] = 0.1*np.ones(2)
        prob['tower_buckling_length'] = 20.0
        prob['tower_outfitting_factor'] = 1.0
        prob['yaw'] = 0.0
        prob['suctionpile_depth'] = 0.0
        prob['soil_G'] = 1e7
        prob['soil_nu'] = 0.5
        prob['E'] = 1e9
        prob['G'] = 1e8
        prob['material_density'] = 1e4
        prob['sigma_y'] = 1e8
        prob['rna_mass'] = 2e5
        prob['rna_I'] = np.r_[1e5, 1e5, 2e5, np.zeros(3)]
        prob['rna_cg'] = np.array([-3., 0.0, 1.0])
        prob['wind_reference_height'] = 80.0
        prob['wind_z0'] = 0.0
        prob['cd_usr'] = -1.
        prob['air_density'] = 1.225
        prob['air_viscosity'] = 1.7934e-5
        prob['water_density'] = 1025.0
        prob['water_viscosity'] = 1.3351e-3
        prob['wind_beta'] = prob['wave_beta'] = 0.0
        prob['significant_wave_height'] = 0.0
        prob['significant_wave_period'] = 1e3
        prob['gamma_f'] = 1.0
        prob['gamma_m'] = 1.0
        prob['gamma_n'] = 1.0
        prob['gamma_b'] = 1.0
        prob['gamma_fatigue'] = 1.0
        prob['DC'] = 80.0
        prob['shear'] = True
        prob['geom'] = True
        prob['tower_force_discretization'] = 5.0
        prob['nM'] = 2
        prob['Mmethod'] = 1
        prob['lump'] = 0
        prob['tol'] = 1e-9
        prob['shift'] = 0.0
        prob['min_d_to_t'] = 120.0
        prob['max_taper'] = 0.2
        prob['wind.Uref'] = 15.0
        prob['pre.rna_F'] = 1e3*np.array([2., 3., 4.,])
        prob['pre.rna_M'] = 1e4*np.array([2., 3., 4.,])
        prob.run_model()

        # All other tests from above
        mass_dens = 1e4*(5.**2-4.9**2)*np.pi
        npt.assert_equal(prob['section_height_out'], prob['tower_section_height'])
        npt.assert_equal(prob['outer_diameter_out'], prob['tower_outer_diameter'])
        npt.assert_equal(prob['wall_thickness_out'], prob['tower_wall_thickness'])
        npt.assert_equal(prob['z_param'], np.array([0., 40., 80.]))
        
        self.assertEqual(prob['height_constraint'], 0.0)
        self.assertEqual(prob['tower_raw_cost'], prob['cm.cost'])
        npt.assert_equal(prob['tower_I_base'], prob['cm.I_base'])
        npt.assert_almost_equal(prob['tower_center_of_mass'], 40.0)
        npt.assert_equal(prob['tower_section_center_of_mass'], prob['cm.section_center_of_mass'])
        self.assertEqual(prob['monopile_mass'], 0.0)
        self.assertEqual(prob['monopile_cost'], 0.0)
        self.assertEqual(prob['monopile_length'], 0.0)
        npt.assert_almost_equal(prob['tower_mass'], mass_dens*80.0)

        npt.assert_equal(prob['pre.kidx'], np.array([0], dtype=np.int_))
        npt.assert_equal(prob['pre.kx'], np.array([1e16]))
        npt.assert_equal(prob['pre.ky'], np.array([1e16]))
        npt.assert_equal(prob['pre.kz'], np.array([1e16]))
        npt.assert_equal(prob['pre.ktx'], np.array([1e16]))
        npt.assert_equal(prob['pre.kty'], np.array([1e16]))
        npt.assert_equal(prob['pre.ktz'], np.array([1e16]))

        npt.assert_equal(prob['pre.midx'], np.array([6, 0, 0]))
        npt.assert_equal(prob['pre.m'], np.array([2e5, 0, 0]))
        npt.assert_equal(prob['pre.mrhox'], np.array([-3., 0., 0.]))
        npt.assert_equal(prob['pre.mrhoy'], np.array([0., 0., 0.]))
        npt.assert_equal(prob['pre.mrhoz'], np.array([1., 0., 0.]))
        npt.assert_equal(prob['pre.mIxx'], np.array([1e5, 0., 0.]))
        npt.assert_equal(prob['pre.mIyy'], np.array([1e5, 0., 0.]))
        npt.assert_equal(prob['pre.mIzz'], np.array([2e5, 0., 0.]))
        npt.assert_equal(prob['pre.mIxy'], np.zeros(3))
        npt.assert_equal(prob['pre.mIxz'], np.zeros(3))
        npt.assert_equal(prob['pre.mIyz'], np.zeros(3))

        npt.assert_equal(prob['pre.plidx'], np.array([6]))
        npt.assert_equal(prob['pre.Fx'], np.array([2e3]))
        npt.assert_equal(prob['pre.Fy'], np.array([3e3]))
        npt.assert_equal(prob['pre.Fz'], np.array([4e3]))
        npt.assert_equal(prob['pre.Mxx'], np.array([2e4]))
        npt.assert_equal(prob['pre.Myy'], np.array([3e4]))
        npt.assert_equal(prob['pre.Mzz'], np.array([4e4]))



    def testProblemFixedPile(self):
        prob = om.Problem()
        prob.model = tow.TowerSE(nLC=1, nPoints=4, nFull=10, wind='PowerWind', topLevelFlag=True, monopile=True)
        prob.setup()

        prob['shearExp'] = 0.2
        prob['hub_height'] = 80.0
        prob['foundation_height'] = -30.0
        prob['transition_piece_height'] = 15.0
        prob['transition_piece_mass'] = 1e2
        prob['gravity_foundation_mass'] = 1e4
        prob['tower_section_height'] = 30.0*np.ones(3)
        prob['tower_outer_diameter'] = 10.0*np.ones(4)
        prob['tower_wall_thickness'] = 0.1*np.ones(3)
        prob['tower_buckling_length'] = 20.0
        prob['tower_outfitting_factor'] = 1.0
        prob['yaw'] = 0.0
        prob['suctionpile_depth'] = 15.0
        prob['soil_G'] = 1e7
        prob['soil_nu'] = 0.5
        prob['E'] = 1e9
        prob['G'] = 1e8
        prob['material_density'] = 1e4
        prob['sigma_y'] = 1e8
        prob['rna_mass'] = 2e5
        prob['rna_I'] = np.r_[1e5, 1e5, 2e5, np.zeros(3)]
        prob['rna_cg'] = np.array([-3., 0.0, 1.0])
        prob['wind_reference_height'] = 80.0
        prob['wind_z0'] = 0.0
        prob['cd_usr'] = -1.
        prob['air_density'] = 1.225
        prob['air_viscosity'] = 1.7934e-5
        prob['water_density'] = 1025.0
        prob['water_viscosity'] = 1.3351e-3
        prob['wind_beta'] = prob['wave_beta'] = 0.0
        prob['significant_wave_height'] = 0.0
        prob['significant_wave_period'] = 1e3
        prob['gamma_f'] = 1.0
        prob['gamma_m'] = 1.0
        prob['gamma_n'] = 1.0
        prob['gamma_b'] = 1.0
        prob['gamma_fatigue'] = 1.0
        prob['DC'] = 80.0
        prob['shear'] = True
        prob['geom'] = True
        prob['tower_force_discretization'] = 5.0
        prob['nM'] = 2
        prob['Mmethod'] = 1
        prob['lump'] = 0
        prob['tol'] = 1e-9
        prob['shift'] = 0.0
        prob['min_d_to_t'] = 120.0
        prob['max_taper'] = 0.2
        prob['wind.Uref'] = 15.0
        prob['pre.rna_F'] = 1e3*np.array([2., 3., 4.,])
        prob['pre.rna_M'] = 1e4*np.array([2., 3., 4.,])
        prob.run_model()

        # All other tests from above
        mass_dens = 1e4*(5.**2-4.9**2)*np.pi
        npt.assert_equal(prob['section_height_out'], np.r_[15., 30.*np.ones(3)])
        npt.assert_equal(prob['outer_diameter_out'], 10.*np.ones(5))
        npt.assert_equal(prob['wall_thickness_out'], 0.1*np.ones(4))
        npt.assert_equal(prob['z_param'], np.array([-45., -30., 0., 30., 60.]))
        
        self.assertEqual(prob['height_constraint'], 20.0)
        self.assertEqual(prob['tower_raw_cost'], (40./105.)*prob['cm.cost'])
        npt.assert_equal(prob['tower_I_base'], prob['cm.I_base'])
        npt.assert_almost_equal(prob['tower_center_of_mass'], (7.5*mass_dens*105.+15.*1e2+1e4*-30.)/(mass_dens*105+1e2+1e4))
        npt.assert_equal(prob['tower_section_center_of_mass'], prob['cm.section_center_of_mass'])
        self.assertEqual(prob['monopile_cost'], (60./105.)*prob['cm.cost'])
        self.assertEqual(prob['monopile_length'], 60.0)
        npt.assert_almost_equal(prob['monopile_mass'], mass_dens*60.0 + 1e2+1e4)
        npt.assert_almost_equal(prob['tower_mass'], mass_dens*45.0)

        npt.assert_equal(prob['pre.kidx'], np.array([0, 1, 2, 3], dtype=np.int_))
        npt.assert_array_less(prob['pre.kx'], 1e16)
        npt.assert_array_less(prob['pre.ky'], 1e16)
        npt.assert_array_less(prob['pre.kz'], 1e16)
        npt.assert_array_less(prob['pre.ktx'], 1e16)
        npt.assert_array_less(prob['pre.kty'], 1e16)
        npt.assert_array_less(prob['pre.ktz'], 1e16)
        npt.assert_array_less(0.0, prob['pre.kx'])
        npt.assert_array_less(0.0, prob['pre.ky'])
        npt.assert_array_less(0.0, prob['pre.kz'])
        npt.assert_array_less(0.0, prob['pre.ktx'])
        npt.assert_array_less(0.0, prob['pre.kty'])
        npt.assert_array_less(0.0, prob['pre.ktz'])

        npt.assert_equal(prob['pre.midx'], np.array([12, 7, 0]))
        npt.assert_equal(prob['pre.m'], np.array([2e5, 1e2, 1e4]))
        npt.assert_equal(prob['pre.mrhox'], np.array([-3., 0., 0.]))
        npt.assert_equal(prob['pre.mrhoy'], np.array([0., 0., 0.]))
        npt.assert_equal(prob['pre.mrhoz'], np.array([1., 0., 0.]))
        npt.assert_equal(prob['pre.mIxx'], np.array([1e5, 1e2*25*0.5, 1e4*25*0.25]))
        npt.assert_equal(prob['pre.mIyy'], np.array([1e5, 1e2*25*0.5, 1e4*25*0.25]))
        npt.assert_equal(prob['pre.mIzz'], np.array([2e5, 1e2*25, 1e4*25*0.5]))
        npt.assert_equal(prob['pre.mIxy'], np.zeros(3))
        npt.assert_equal(prob['pre.mIxz'], np.zeros(3))
        npt.assert_equal(prob['pre.mIyz'], np.zeros(3))

        npt.assert_equal(prob['pre.plidx'], np.array([12]))
        npt.assert_equal(prob['pre.Fx'], np.array([2e3]))
        npt.assert_equal(prob['pre.Fy'], np.array([3e3]))
        npt.assert_equal(prob['pre.Fz'], np.array([4e3]))
        npt.assert_equal(prob['pre.Mxx'], np.array([2e4]))
        npt.assert_equal(prob['pre.Myy'], np.array([3e4]))
        npt.assert_equal(prob['pre.Mzz'], np.array([4e4]))
        
        

    def testAddedMassForces(self):
        prob = om.Problem()
        prob.model = tow.TowerSE(nLC=1, nPoints=4, nFull=10, wind='PowerWind', topLevelFlag=True, monopile=True)
        prob.setup()

        prob['shearExp'] = 0.2
        prob['hub_height'] = 80.0
        prob['foundation_height'] = -30.0
        prob['transition_piece_height'] = 15.0
        prob['transition_piece_mass'] = 0.0
        prob['gravity_foundation_mass'] = 0.0
        prob['tower_section_height'] = 30.0*np.ones(3)
        prob['tower_outer_diameter'] = 10.0*np.ones(4)
        prob['tower_wall_thickness'] = 0.1*np.ones(3)
        prob['tower_buckling_length'] = 20.0
        prob['tower_outfitting_factor'] = 1.0
        prob['yaw'] = 0.0
        prob['suctionpile_depth'] = 15.0
        prob['soil_G'] = 1e7
        prob['soil_nu'] = 0.5
        prob['E'] = 1e9
        prob['G'] = 1e8
        prob['material_density'] = 1e4
        prob['sigma_y'] = 1e8
        prob['rna_mass'] = 0.0
        prob['rna_I'] = np.r_[1e5, 1e5, 2e5, np.zeros(3)]
        prob['rna_cg'] = np.array([-3., 0.0, 1.0])
        prob['wind_reference_height'] = 80.0
        prob['wind_z0'] = 0.0
        prob['cd_usr'] = -1.
        prob['air_density'] = 1.225
        prob['air_viscosity'] = 1.7934e-5
        prob['water_density'] = 1025.0
        prob['water_viscosity'] = 1.3351e-3
        prob['wind_beta'] = prob['wave_beta'] = 0.0
        prob['significant_wave_height'] = 0.0
        prob['significant_wave_period'] = 1e3
        prob['gamma_f'] = 1.0
        prob['gamma_m'] = 1.0
        prob['gamma_n'] = 1.0
        prob['gamma_b'] = 1.0
        prob['gamma_fatigue'] = 1.0
        prob['DC'] = 80.0
        prob['shear'] = True
        prob['geom'] = True
        prob['tower_force_discretization'] = 5.0
        prob['nM'] = 2
        prob['Mmethod'] = 1
        prob['lump'] = 0
        prob['tol'] = 1e-9
        prob['shift'] = 0.0
        prob['min_d_to_t'] = 120.0
        prob['max_taper'] = 0.2
        prob['wind.Uref'] = 15.0
        prob['pre.rna_F'] = 1e3*np.array([2., 3., 4.,])
        prob['pre.rna_M'] = 1e4*np.array([2., 3., 4.,])
        prob.run_model()

        myFz = copy.copy(prob['post.Fz'])

        prob['rna_mass'] = 1e4
        prob.run_model()
        myFz[3:] -= 1e4*g
        npt.assert_almost_equal(prob['post.Fz'], myFz)

        prob['transition_piece_mass'] = 1e2
        prob.run_model()
        myFz[3:7] -= 1e2*g
        npt.assert_almost_equal(prob['post.Fz'], myFz)

        prob['gravity_foundation_mass'] = 1e3
        prob.run_model()
        #myFz[3] -= 1e3*g
        npt.assert_almost_equal(prob['post.Fz'], myFz)
        
def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestTowerSE))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())



"""`InstallPhase` base class."""

__author__ = ["Jake Nunemaker", "Rob Hammond"]
__copyright__ = "Copyright 2020, National Renewable Energy Laboratory"
__maintainer__ = ["Jake Nunemaker", "Rob Hammond"]
__email__ = ["jake.nunemaker@nrel.gov", "robert.hammond@nrel.gov"]


from abc import abstractmethod
from itertools import groupby

import numpy as np
import simpy

from wisdem.orbit.core import Port, Environment
from wisdem.orbit.phases import BasePhase


class InstallPhase(BasePhase):
    """BasePhase subclass for install modules."""

    def __init__(self, weather, **kwargs):
        """
        Creates an instance of `InstallPhase`.

        Parameters
        ----------
        weather : np.ndarray
            Weather profile at site.
        """

        self.extract_phase_kwargs(**kwargs)
        self.initialize_environment(weather, **kwargs)

    def initialize_environment(self, weather, **kwargs):
        """
        Initializes a `marmot.Environment` at `self.env`.

        Parameters
        ----------
        weather : np.ndarray
            Weather profile at site.
        """

        env_name = kwargs.get("env_name", "Environment")
        self.env = Environment(name=env_name, state=weather, **kwargs)

    @abstractmethod
    def setup_simulation(self):
        """
        Sets up the required simulation infrastructure

        Generally, this creates the port, initializes the items to be
        installed, and initializes the vessel(s) used for the installation.
        """

        pass

    def initialize_port(self):
        """
        Initializes a Port object with N number of cranes.
        """

        try:
            cranes = self.config["port"]["num_cranes"]
            self.port = Port(self.env)
            self.port.crane = simpy.Resource(self.env, cranes)

        except KeyError:
            self.port = Port(self.env)

    def run(self, until=None):
        """
        Runs the simulation on self.env.

        Parameters
        ----------
        until : int, optional
            Number of steps to run.
        """

        self.env._submit_log({"message": "SIMULATION START"}, "DEBUG")
        self.env.run(until=until)
        self.append_phase_info()
        self.env._submit_log({"message": "SIMULATION END"}, "DEBUG")

    def append_phase_info(self):
        """Appends phase information to all logs in `self.env.logs`."""

        for l in self.env.logs:
            l["phase"] = self.phase

    @property
    def port_costs(self):
        """Cost of port rental."""

        port = getattr(self, "port", None)

        if port is None:
            return 0

        else:
            key = "port_cost_per_month"
            rate = self.config["port"].get("monthly_rate", self.defaults[key])

            months = self.total_phase_time / (8760 / 12)
            return months * rate

    @property
    def total_phase_cost(self):
        """Returns total phase cost in $USD."""

        return self.action_costs + self.port_costs

    @property
    def action_costs(self):
        """Returns sum cost of all actions."""

        return np.nansum([a["cost"] for a in self.env.actions])

    @property
    def total_phase_time(self):
        """Returns total phase time in hours."""

        return max([a["time"] for a in self.env.actions])

    @property
    @abstractmethod
    def detailed_output(self):
        """Returns detailed phase information."""

        pass

    @property
    def agent_efficiencies(self):
        """
        Returns a summary of agent operational efficiencies.
        """

        efficiencies = {}

        s = sorted(self.env.actions, key=lambda x: (x["agent"], x["action"]))
        grouped = {
            k: sum([i["duration"] for i in list(v)])
            for k, v in groupby(s, key=lambda x: (x["agent"], x["action"]))
        }
        agents = list(set([k[0] for k in grouped.keys()]))
        for agent in agents:
            total = sum([v for k, v in grouped.items() if k[0] == agent])

            try:
                delay = grouped[(agent, "Delay")]
                e = (total - delay) / total

            except KeyError:
                delay = 0.0
                e = 1.0

            except ZeroDivisionError:
                e = 1.0

            if not 0.0 <= e <= 1.0:
                raise ValueError(f"Invalid efficiency for agent '{agent}'")

            name = str(agent).replace(" ", "_")
            efficiencies[f"{name}_operational_efficiency"] = e

        return efficiencies

    @staticmethod
    def get_max_cargo_mass_utilzations(vessels):
        """
        Returns a summary of cargo mass efficiencies for list of input `vessels`.

        Parameters
        ----------
        vessels : list
            List of vessels to calculate efficiencies for.
        """

        outputs = {}

        for vessel in vessels:
            name = vessel.name.replace(" ", "_")
            storage = getattr(vessel, "storage", None)
            if storage is None:
                print("Vessel does not have storage capacity.")
                continue

            outputs[
                f"{name}_cargo_mass_utilization"
            ] = vessel.max_cargo_mass_utilization

        return outputs

    @staticmethod
    def get_max_deck_space_utilzations(vessels):
        """
        Returns a summary of deck space efficiencies for list of input `vessels`.

        Parameters
        ----------
        vessels : list
            List of vessels to calculate efficiencies for.
        """

        outputs = {}

        for vessel in vessels:
            name = vessel.name.replace(" ", "_")
            storage = getattr(vessel, "storage", None)
            if storage is None:
                print("Vessel does not have storage capacity.")
                continue

            outputs[
                f"{name}_deck_space_utilization"
            ] = vessel.max_deck_space_utilization

        return outputs



from __future__ import print_function

import numpy as np
import numpy.testing as npt
import unittest
import time
import wisdem.floatingse.loading as sP
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

from wisdem.commonse import gravity as g


NSECTIONS = 5
NPTS = NSECTIONS+1

def DrawTruss(mytruss):
    mynodes = {}
    for k in range(len(mytruss.myframe.nx)):
        mynodes[mytruss.myframe.nnode[k]] = np.r_[mytruss.myframe.nx[k], mytruss.myframe.ny[k], mytruss.myframe.nz[k]]
    myelem = []
    for k in range(len(mytruss.myframe.eN1)):
        myelem.append( (mytruss.myframe.eN1[k], mytruss.myframe.eN2[k]) )

    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    for e in myelem:
        xs = np.array( [ mynodes[e[0]][0], mynodes[e[1]][0] ] )
        ys = np.array( [ mynodes[e[0]][1], mynodes[e[1]][1] ] )
        zs = np.array( [ mynodes[e[0]][2], mynodes[e[1]][2] ] )
        ax.plot(xs, ys, zs)
    #ax.auto_scale_xyz([-10, 10], [-10, 10], [-30, 50])
    plt.show()


def getInputs():
    inputs = {}
    discrete_inputs = {}

    inputs['E'] = 200e9
    inputs['G'] = 79.3e9
    inputs['material_density'] = 7850.0
    inputs['yield_stress'] = 345e6
    inputs['Hs'] = 10.0

    inputs['water_density'] = 1025.0

    discrete_inputs['cross_attachment_pontoons'] = True
    discrete_inputs['lower_attachment_pontoons'] = True
    discrete_inputs['upper_attachment_pontoons'] = True
    discrete_inputs['lower_ring_pontoons'] = True
    discrete_inputs['upper_ring_pontoons'] = True
    discrete_inputs['outer_cross_pontoons'] = True

    inputs['number_of_offset_columns'] = 3
    inputs['connection_ratio_max'] = 0.25

    inputs['number_of_mooring_connections'] = 3
    inputs['mooring_lines_per_connection'] = 1
    inputs['mooring_neutral_load'] = 1e2*np.ones((15,3))
    inputs['mooring_stiffness'] = 1e10 * np.eye(6)
    inputs['mooring_moments_of_inertia'] = np.ones(6)
    
    inputs['fairlead'] = 7.5
    inputs['fairlead_radius'] = 1.0
    inputs['fairlead_support_outer_diameter'] = 2.0
    inputs['fairlead_support_wall_thickness'] = 1.0
    
    inputs['main_pontoon_attach_upper'] = 1.0
    inputs['main_pontoon_attach_lower'] = 0.0

    inputs['radius_to_offset_column'] = 15.0
    inputs['pontoon_outer_diameter'] = 2.0
    inputs['pontoon_wall_thickness'] = 1.0

    inputs['main_z_full'] = np.array([-15.0, -12.5, -10.0, 0.0, 5.0, 10.0])
    inputs['main_d_full'] = 2*10.0 * np.ones(NPTS)
    inputs['main_t_full'] = 0.1 * np.ones(NPTS-1)
    inputs['main_mass'] = 1e2 * np.ones(NSECTIONS)
    inputs['main_buckling_length'] = 2.0 * np.ones(NSECTIONS)
    inputs['main_displaced_volume'] = 1e2 * np.ones(NSECTIONS)
    inputs['main_hydrostatic_force'] = np.zeros(NSECTIONS)
    inputs['main_hydrostatic_force'][0] = inputs['main_displaced_volume'].sum()*g*inputs['water_density']
    inputs['main_center_of_buoyancy'] = -10.0
    inputs['main_center_of_mass'] = -6.0
    inputs['main_Px'] = 50.0 * np.ones(NPTS)
    inputs['main_Py'] = np.zeros(NPTS)
    inputs['main_Pz'] = np.zeros(NPTS)
    inputs['main_qdyn'] = 70.0 * np.ones(NPTS)

    inputs['offset_z_full'] = np.array([-15.0, -10.0, -5.0, 0.0, 2.5, 10.0])
    inputs['offset_d_full'] = 2*2.0 * np.ones(NPTS)
    inputs['offset_t_full'] = 0.05 * np.ones(NPTS-1)
    inputs['offset_mass'] = 1e1 * np.ones(NSECTIONS)
    inputs['offset_buckling_length'] = 2.0 * np.ones(NSECTIONS)
    inputs['offset_displaced_volume'] = 1e1 * np.ones(NSECTIONS)
    inputs['offset_hydrostatic_force'] = np.zeros(NSECTIONS)
    inputs['offset_hydrostatic_force'][0] = inputs['main_displaced_volume'].sum()*g*inputs['water_density']
    inputs['offset_center_of_buoyancy'] = -5.0
    inputs['offset_center_of_mass'] = -3.0
    inputs['offset_Px'] = 50.0 * np.ones(NPTS)
    inputs['offset_Py'] = np.zeros(NPTS)
    inputs['offset_Pz'] = np.zeros(NPTS)
    inputs['offset_qdyn'] = 70.0 * np.ones(NPTS)

    inputs['tower_z_full'] = np.linspace(0, 90, NPTS)
    inputs['tower_d_full'] = 2*7.0 * np.ones(NPTS)
    inputs['tower_t_full'] = 0.5 * np.ones(NPTS-1)
    inputs['tower_mass_section'] = 2e2 * np.ones(NSECTIONS)
    inputs['tower_buckling_length'] = 25.0
    inputs['tower_center_of_mass'] = 50.0
    inputs['tower_Px'] = 50.0 * np.ones(NPTS)
    inputs['tower_Py'] = np.zeros(NPTS)
    inputs['tower_Pz'] = np.zeros(NPTS)
    inputs['tower_qdyn'] = 70.0 * np.ones(NPTS)

    inputs['rna_force'] = 6e1*np.ones(3)
    inputs['rna_moment'] = 7e2*np.ones(3)
    inputs['rna_mass'] = 6e1
    inputs['rna_cg'] = np.array([3.05, 2.96, 2.13])
    inputs['rna_I'] = np.array([3.05284574e9, 2.96031642e9, 2.13639924e7, 0.0, 2.89884849e7, 0.0])

    inputs['gamma_f'] = 1.35
    inputs['gamma_m'] = 1.1
    inputs['gamma_n'] = 1.0
    inputs['gamma_b'] = 1.1
    inputs['gamma_fatigue'] = 1.755

    inputs['material_cost_rate'] = 1.0
    inputs['painting_cost_rate'] = 10.0
    inputs['labor_cost_rate'] = 2.0
    return inputs, discrete_inputs

    
class TestFrame(unittest.TestCase):
    def setUp(self):
        self.inputs, self.discrete_inputs = getInputs()
        self.outputs = {}
        self.discrete_outputs = {}

        self.outputs['pontoon_stress'] = np.zeros(70)
        
        self.mytruss = sP.FloatingFrame(nFull=NPTS,nFullTow=NPTS)

    def tearDown(self):
        self.mytruss = None

    def testStandard(self):
        self.inputs['radius_to_offset_column'] = 20.0
        self.inputs['fairlead_radius'] = 30.0
        self.inputs['offset_z_full'] = np.array([-15.0, -10.0, -5.0, 0.0, 2.5, 3.0])
        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)

        npt.assert_equal(self.outputs['main_connection_ratio'], 0.25-0.1)
        npt.assert_equal(self.outputs['offset_connection_ratio'], 0.25-0.5)
        #DrawTruss(self.mytruss)
        

    def testSpar(self):
        self.inputs['cross_attachment_pontoons'] = False
        self.inputs['lower_attachment_pontoons'] = False
        self.inputs['upper_attachment_pontoons'] = False
        self.inputs['lower_ring_pontoons'] = False
        self.inputs['upper_ring_pontoons'] = False
        self.inputs['outer_cross_pontoons'] = False
        self.inputs['number_of_offset_columns'] = 0
        
        #self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)

        #DrawTruss(self.mytruss)
        
        
    def testOutputsIncremental(self):
        ncyl   = self.inputs['number_of_offset_columns']
        R_semi = self.inputs['radius_to_offset_column']
        Rmain  = 0.5*self.inputs['main_d_full'][0]
        Roff   = 0.5*self.inputs['offset_d_full'][0]
        Ro     = 0.5*self.inputs['pontoon_outer_diameter']
        Ri     = Ro - self.inputs['pontoon_wall_thickness']
        rho    = self.inputs['material_density']
        rhoW   = self.inputs['water_density']

        self.inputs['fairlead_radius'] = 0.0
        self.inputs['fairlead_outer_diameter'] = 0.0
        self.inputs['fairlead_wall_thickness'] = 0.0
        self.inputs['mooring_lines_per_column'] = 0.0
        self.inputs['number_of_mooring_connections'] = 0.0
        self.discrete_inputs['cross_attachment_pontoons'] = False
        self.discrete_inputs['lower_attachment_pontoons'] = True
        self.discrete_inputs['upper_attachment_pontoons'] = False
        self.discrete_inputs['lower_ring_pontoons'] = False
        self.discrete_inputs['upper_ring_pontoons'] = False
        self.discrete_inputs['outer_cross_pontoons'] = False
        self.inputs['pontoon_main_attach_upper'] = 1.0
        self.inputs['pontoon_main_attach_lower'] = 0.0
        self.inputs['material_cost_rate'] = 6.25
        self.inputs['painting_cost_rate'] = 0.0
        self.inputs['labor_cost_rate'] = 0.0

        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)
        V = np.pi * Ro*Ro * (R_semi-Rmain-Roff) * ncyl
        m = np.pi * (Ro*Ro-Ri*Ri) * (R_semi-Rmain-Roff) * ncyl * rho
        self.assertAlmostEqual(self.outputs['pontoon_displacement'], V)
        self.assertAlmostEqual(self.outputs['pontoon_center_of_buoyancy'], -15.0)
        self.assertAlmostEqual(self.outputs['pontoon_center_of_mass'], -15.0)
        self.assertAlmostEqual(self.outputs['pontoon_mass'], m)
        self.assertAlmostEqual(self.outputs['pontoon_cost'], m*6.25, 2)


        self.discrete_inputs['lower_ring_pontoons'] = True
        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)

        V = np.pi * Ro*Ro * ncyl * (R_semi * (1 + np.sqrt(3)) - Rmain - 3*Roff)
        m = np.pi * (Ro*Ro-Ri*Ri) * ncyl * rho * (R_semi * (1 + np.sqrt(3)) - Rmain - 3*Roff)
        self.assertAlmostEqual(self.outputs['pontoon_displacement'], V)
        self.assertAlmostEqual(self.outputs['pontoon_center_of_buoyancy'], -15.0)
        self.assertAlmostEqual(self.outputs['pontoon_center_of_mass'], -15.0)
        self.assertAlmostEqual(self.outputs['pontoon_mass'], m)
        self.assertAlmostEqual(self.outputs['pontoon_cost'], m*6.25, 2)


        self.discrete_inputs['upper_attachment_pontoons'] = True # above waterline
        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)

        V = np.pi * Ro*Ro * ncyl * (R_semi * (1 + np.sqrt(3)) - Rmain - 3*Roff)
        m = np.pi * (Ro*Ro-Ri*Ri) * ncyl * rho * (R_semi * (2 + np.sqrt(3)) - 2*Rmain - 4*Roff)
        #cg = ((-15)*(1 + np.sqrt(3)) + 10) / (2+np.sqrt(3))
        self.assertAlmostEqual(self.outputs['pontoon_displacement'], V)
        self.assertAlmostEqual(self.outputs['pontoon_center_of_buoyancy'], -15.0)
        #self.assertAlmostEqual(self.outputs['pontoon_center_of_mass'], cg)
        self.assertAlmostEqual(self.outputs['pontoon_mass'], m)
        self.assertAlmostEqual(self.outputs['pontoon_cost'], m*6.25, 2)

        
        self.discrete_inputs['upper_ring_pontoons'] = True
        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)

        V = np.pi * Ro*Ro * ncyl * (R_semi * (1 + np.sqrt(3)) - Rmain - 3*Roff)
        m = np.pi * (Ro*Ro-Ri*Ri) * ncyl * rho * 2 * (R_semi * (1 + np.sqrt(3)) - Rmain - 3*Roff)
        self.assertAlmostEqual(self.outputs['pontoon_displacement'], V)
        self.assertAlmostEqual(self.outputs['pontoon_center_of_buoyancy'], -15.0)
        self.assertAlmostEqual(self.outputs['pontoon_center_of_mass'], -2.5)
        self.assertAlmostEqual(self.outputs['pontoon_mass'], m)
        self.assertAlmostEqual(self.outputs['pontoon_cost'], m*6.25, 2)

        
        #self.inputs['cross_attachment_pontoons'] = True
        #self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)

        #L = np.sqrt(R_semi*R_semi + 25*25)
        #k = 15. / 25.
        #V = np.pi * Ro*Ro * ncyl * (k*L + R_semi * (1 + np.sqrt(3)))
        #m = np.pi * (Ro*Ro-Ri*Ri) * ncyl * rho * (L + R_semi * 2 * (1 + np.sqrt(3)))
        #self.assertAlmostEqual(self.outputs['pontoon_displacement'], V)
        #self.assertAlmostEqual(self.outputs['pontoon_mass'], m)
        #self.assertAlmostEqual(self.outputs['pontoon_cost'], m*6.25, 2)

        
        #self.inputs['outer_cross_pontoons'] = True
        #self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)


    def testForces(self):
        self.inputs['offset_z_full'] = np.linspace(-1e-4, 0.0, NPTS)
        self.inputs['main_z_full'] = np.linspace(-1e-4, 0.0, NPTS)
        self.inputs['tower_z_full'] = np.linspace(0, 1e-4, NPTS)
        self.inputs['fairlead'] = 0.0
        self.inputs['number_of_offset_columns'] = 0
        self.inputs['main_mass'] = 1.0 * np.ones(NSECTIONS)
        self.inputs['main_center_of_mass'] = 0.0
        self.inputs['main_hydrostatic_force'] = 1e-12 * np.ones(NSECTIONS)
        self.inputs['offset_mass'] = 0.0 * np.ones(NSECTIONS)
        self.inputs['offset_center_of_mass'] = 0.0
        self.inputs['offset_hydrostatic_force'] = 1e-12 * np.ones(NSECTIONS)
        self.inputs['tower_mass_section'] = 1.0 * np.ones(NSECTIONS)
        self.inputs['tower_center_of_mass'] = 0.0
        self.inputs['rna_mass'] = 1.0
        self.inputs['rna_force'] = 10.0*np.ones(3)
        self.inputs['rna_moment'] = 20.0*np.ones(3)
        self.inputs['rna_cg'] = np.array([0.0, 0.0, 0.0])
        self.inputs['rna_I'] = np.zeros(6)
        self.inputs['main_Px'] = 0.0 * np.ones(NSECTIONS)
        self.inputs['offset_Px'] = 0.0 * np.ones(NSECTIONS)
        self.inputs['tower_Px'] = 0.0 * np.ones(NSECTIONS)
        self.inputs['main_qdyn'] = 0.0 * np.ones(NPTS)
        self.inputs['offset_qdyn'] = 0.0 * np.ones(NPTS)
        self.inputs['tower_qdyn'] = 0.0 * np.ones(NPTS)
        self.discrete_inputs['cross_attachment_pontoons'] = False
        self.discrete_inputs['lower_attachment_pontoons'] = False
        self.discrete_inputs['upper_attachment_pontoons'] = False
        self.discrete_inputs['lower_ring_pontoons'] = False
        self.discrete_inputs['upper_ring_pontoons'] = False
        self.discrete_inputs['outer_cross_pontoons'] = False
        self.inputs['main_pontoon_attach_upper'] = 1.0
        self.inputs['main_pontoon_attach_lower'] = 0.0
        self.inputs['water_density'] = 1e-12
        self.inputs['number_of_mooring_connections'] = 3
        self.inputs['mooring_lines_per_connection'] = 1
        self.inputs['mooring_neutral_load'] = 10.0*np.ones((15,3))
        self.inputs['mooring_moments_of_inertia'] = np.ones(6)
        self.inputs['fairlead_radius'] = 10.1
        self.inputs['fairlead_support_outer_diameter'] = 2*np.sqrt(2.0/np.pi)
        self.inputs['fairlead_support_wall_thickness'] = np.sqrt(2.0/np.pi) - np.sqrt(1.0/np.pi)
        self.inputs['material_density'] = 20.0
        self.inputs['radius_to_offset_column'] = 1.0

        goodRun = False
        kiter = 0
        while goodRun == False:
            kiter += 1
            self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)
            if self.outputs['substructure_mass'] < 1e3:
                goodRun = True
            if kiter > 3:
                self.assertTrue(goodRun)
            
        msub = NSECTIONS + 3*20.0*1.0*0.1
        mtowrna = NSECTIONS + 1
        mtot = msub + mtowrna

        self.assertAlmostEqual(self.outputs['substructure_mass'], msub, 5)
        self.assertAlmostEqual(self.outputs['structural_mass'], mtot, 4)
        npt.assert_almost_equal(self.outputs['total_force'], 10*3 + np.array([10.0, 10.0, 10-mtot*g]), decimal=1)
        npt.assert_almost_equal(self.outputs['total_moment'], np.array([20.0, 20.0, 20.0]), decimal=0)

        self.inputs['rna_cg'] = np.array([5.0, 5.0, 5.0])
        goodRun = False
        kiter = 0
        while goodRun == False:
            kiter += 1
            self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)
            if self.outputs['substructure_mass'] < 1e3:
                goodRun = True
            if kiter > 3:
                self.assertTrue(goodRun)
        npt.assert_almost_equal(self.outputs['total_force'], 3*10 + np.array([10.0, 10.0, 10-mtot*g]), decimal=1)
        #self.assertAlmostEqual(self.outputs['total_moment'][-1], 20.0)
        
class TestSandbox(unittest.TestCase):
    def setUp(self):
        self.inputs, self.discrete_inputs = getInputs()
        self.outputs = {}
        self.discrete_outputs = {}
        self.outputs['pontoon_stress'] = np.zeros(70)
        self.mytruss = sP.FloatingFrame(nFull=NPTS,nFullTow=NPTS)

    def tearDown(self):
        self.mytruss = None

        
    def testBadInput(self):
        self.inputs['number_of_offset_columns'] = 1
        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)
        self.assertEqual(self.outputs['substructure_mass'], 1e30)

        self.inputs['number_of_offset_columns'] = 2
        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)
        self.assertEqual(self.outputs['substructure_mass'], 1e30)

        self.inputs['number_of_offset_columns'] = 8
        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)
        self.assertEqual(self.outputs['substructure_mass'], 1e30)

        self.inputs['number_of_offset_columns'] = 3
        self.inputs['main_z_full'][-2] = self.inputs['main_z_full'][-3] + 1e-12
        self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)
        self.assertEqual(self.outputs['substructure_mass'], 1e30)

    def testCombinations(self):
        self.inputs['radius_to_offset_column'] = 30.0
        
        for nc in [0, 3, 4, 5, 6, 7]:
            
            for cap in [True, False]:
                
                for lap in [True, False]:
                    
                    for uap in [True, False]:
                        
                        for lrp in [True, False]:
                            
                            for urp in [True, False]:
                                
                                for ocp in [True, False]:
                                    self.inputs['number_of_offset_columns'] = nc
                                    self.inputs['number_of_mooring_connections'] = np.maximum(nc, 3)
                                    self.inputs['mooring_lines_per_connection'] = 1
                                    self.discrete_inputs['cross_attachment_pontoons'] = cap
                                    self.discrete_inputs['lower_attachment_pontoons'] = lap
                                    self.discrete_inputs['upper_attachment_pontoons'] = uap
                                    self.discrete_inputs['lower_ring_pontoons'] = lrp
                                    self.discrete_inputs['upper_ring_pontoons'] = urp
                                    self.discrete_inputs['outer_cross_pontoons'] = ocp
                                    if (nc > 0) and (not cap) and (not lap) and (not uap): continue
                                    if (nc > 0) and (ocp) and (not lrp): continue
                                    
                                    self.mytruss.compute(self.inputs, self.outputs, self.discrete_inputs, self.discrete_outputs)
                                    if self.outputs['substructure_mass'] == 1e30:
                                        print(nc, cap, lap, uap, lrp, urp, ocp)
                                    self.assertNotEqual(self.outputs['substructure_mass'], 1e30)
                                    time.sleep(1e-3)

        
def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestFrame))
    suite.addTest(unittest.makeSuite(TestSandbox))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())




import os
import sys

def under_mpirun():
    """Return True if we're being executed under mpirun."""
    # this is a bit of a hack, but there appears to be
    # no consistent set of environment vars between MPI
    # implementations.
    for name in os.environ.keys():
        if name == 'OMPI_COMM_WORLD_RANK' or \
           name == 'MPIEXEC_HOSTNAME' or \
           name.startswith('MPIR_') or \
           name.startswith('MPICH_'):
            return True
    return False

if under_mpirun():
    from mpi4py import MPI

    def debug(*msg):  # pragma: no cover
        newmsg = ["%d: " % MPI.COMM_WORLD.rank] + list(msg)
        for m in newmsg:
            sys.stdout.write("%s " % m)
        sys.stdout.write('\n')
        sys.stdout.flush()
else:
    MPI = None


#-------------------------------------------------------------------------------
# Name:        Material.py
# Purpose: This module contains the material class definition
#
# Author:      rdamiani
#
# Created:     04/11/2013
# Copyright:   (c) rdamiani 2013
# Licence:     APache (2014)
#-------------------------------------------------------------------------------

def main():
    """Material Class"""
    #The main fct is written to test the class
    mat=Material(E=3.5e5,rho=8500.)
    mat1=Material(matname='grout')
    print('Example Returning mat and mat1 as 2 objects')
    return mat,mat1


class Material: #isotropic for the time being
    def __init__(self,**kwargs):

        prms={'matname':'ASTM992_steel','E':2.1e11,'nu':0.33, 'rho':8502., 'fy':345.e6, 'fyc':345.e6, 'fp':0.0, 'relx':0.0} #SI Units
        prms.update(kwargs) #update in case user put some new params in
        for key in prms:  #Initialize material object with these parameters, possibly updated by user' stuff
            setattr(self,key,prms[key])

        #Fill in the rest of the fields user may have skipped

        #Predefined materials
        steel={'matname':'ASTM992_steel',          'E':2.1e11,'nu':0.33,'G':7.895e10,'rho':7805., 'fy':345.e6, 'fyc':345.e6,} #SI Units
        heavysteel={'matname':'ASTM992_steelheavy','E':2.1e11,'nu':0.33,'G':7.895e10,'rho':8741., 'fy':345.e6, 'fyc':345.e6} #SI Units
        grout={'matname':'Grout',                  'E':3.9e10,'nu':0.33,'G':1.466e10,'rho':2500., 'fy':20.68e6,'fyc':20.68e6} #SI Units  TO REVISE FOR GROUT
        concrete={'matname':'Concrete',            'E':24.8e9,'nu':0.18,             'rho':2300,  'fy':0.,     'fyc':27.6e6} #SI Units
        rebar={'matname':'Reinforcement',          'E':200e9, 'nu':0.33,             'rho':7805,  'fy':276.e6, 'fyc':276.e6} #SI Units
        strand={'matname':"Strand",                'E':195e9, 'nu':0.33,             'rho':7805,  'fy':1690.e6, 'fyc':1690.e6, 'fp':1930.e6, 'relx':67} #SI Units;fp=ultimate stress; relx is for Roo, relaxation calculation

        if ((prms['matname'].lower() == 'steel') or (prms['matname'].lower() == 'astm992_steel')  or (prms['matname'].lower() == '')):
            for key in prms:
                if (not(key in kwargs) or (prms[key]==[])) and (key in steel):  #I need to operate on parameters not set by user
                    setattr(self,key,steel[key])

        elif ((prms['matname'].lower() == 'heavysteel') or (prms['matname'].lower() == 'astm992_steelheavy') or (prms['matname'].lower() == 'heavysteel')):
            for key in prms:
                if (not(key in kwargs) or (prms[key]==[])) and (key in heavysteel):  #I need to operate on parameters not set by user
                    setattr(self,key,heavysteel[key])

        elif ( (prms['matname'].lower() == 'grout')):
            for key in prms:
                if (not(key in kwargs) or (prms[key]==[])) and (key in grout):
                    setattr(self,key,grout[key])

        elif prms['matname'].lower() == 'concrete':
            for key in prms:
                if (not(key in kwargs) or (prms[key]==[])) and (key in concrete):
                    setattr(self,key,concrete[key])

        elif ((prms['matname'].lower() == 'rebar') or (prms['matname'].lower() == 'reinforcement')):
            for key in prms:
                if (not(key in kwargs) or (prms[key]==[])) and (key in rebar): #I need to operate on parameters not set by user
                    setattr(self,key,rebar[key])

        elif ((prms['matname'].lower() == 'strand') or (prms['matname'].lower() == 'tendon')):
            for key in prms:
                if (not(key in kwargs) or (prms[key]==[])) and (key in strand):  #I need to operate on parameters not set by user
                    setattr(self,key,strand[key])

        if not(hasattr(self,'G')) or not(self.G):
            self.G=self.E/(2.*(1.+self.nu))  #isotropic
        else: #if G is given then recalculate nu
            self.nu=self.E/(2.*self.G)-1.

if __name__ == '__main__':
    mat,mat1=main()



import numpy as np
import numpy.testing as npt
import unittest
import wisdem.commonse.utilities as util

npts = 100
myones = np.ones((npts,))

class TestAny(unittest.TestCase):

            
    def testNodal2Sectional(self):
        x,dx = util.nodal2sectional(np.array([8.0, 10.0, 12.0]))
        npt.assert_equal(x, np.array([9.0, 11.0]))
        npt.assert_equal(dx, np.array([[0.5, 0.5, 0.0],[0.0, 0.5, 0.5]]))


    def testSectionalInterp(self):
        x = np.arange(0.0, 2.1, 0.5)
        y = np.array([-1.0, 1.0, -2.0, 2.0])
        xi = np.array([-0.1, 0.25, 0.9, 1.4, 1.5, 1.6, 2.1])
        yi = util.sectionalInterp(xi, x, y)

        y_expect = np.array([-1.0, -1.0, 1.0, -2.0, 2.0, 2.0, 2.0])
        npt.assert_array_equal(yi, y_expect)
        
        
def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestAny))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())



import unittest
import wisdem.commonse.enum as enum

class TestEnum(unittest.TestCase):
    def setUp(self):
        self.myenum = enum.Enum("One two THREE")

    def testValues(self):
        self.assertEqual(self.myenum.ONE, 0)
        self.assertEqual(self.myenum.TWO, 1)
        self.assertEqual(self.myenum.THREE, 2)

    def testValueFromString(self):
        self.assertEqual(self.myenum['ONE'], 0)
        self.assertEqual(self.myenum['TWO'], 1)
        self.assertEqual(self.myenum['THREE'], 2)

    def testValueFromUnicode(self):
        self.assertEqual(self.myenum[u'ONE'], 0)
        self.assertEqual(self.myenum[u'TWO'], 1)
        self.assertEqual(self.myenum[u'THREE'], 2)

    def testLen(self):
        self.assertEqual( len(self.myenum), 3)

    def testReverseLookup(self):
        self.assertEqual( self.myenum[0], 'ONE')
        self.assertEqual( self.myenum[1], 'TWO')
        self.assertEqual( self.myenum[2], 'THREE')
        self.assertEqual( self.myenum[-1], 'THREE')


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestEnum))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())
